@@ -101,8 +101,9 @@ class CondaStore(LoggingConfigurable):
         [
             "main",
             "conda-forge",
+            "https://repo.anaconda.com/pkgs/main",
         ],
-        help="Allowed conda channels to be used in conda environments",
+        help="Allowed conda channels to be used in conda environments. If set to empty list all channels are accepted. Defaults to main and conda-forge",
         config=True,
     )
 
@@ -154,8 +155,16 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    upgrade_db = Bool(
+        True,
+        help="""Upgrade the database automatically on start.
+        Only safe if database is regularly backed up.
+        """,
+        config=True,
+    )
+
     redis_url = Unicode(
-        help="Redis connection url in form 'redis://:<password>@<hostname>:<port>/0'. Connection is used by Celery along with Conda-Store internally",
+        help="Redis connection url in form 'redis://:<password>@<hostname>:<port>/0'. Connection is used by Celery along with conda-store internally",
         config=True,
     )
 
@@ -324,17 +333,6 @@ class CondaStore(LoggingConfigurable):
             },
         }
 
-        if self.celery_results_backend.startswith("sqla"):
-            # https://github.com/celery/celery/issues/4653#issuecomment-400029147
-            # race condition in table construction in celery
-            # despite issue being closed still causes first task to fail
-            # in celery if tables not created
-            from celery.backends.database import SessionManager
-
-            session = SessionManager()
-            engine = session.get_engine(self._celery_app.backend.url)
-            session.prepare_models(engine)
-
         return self._celery_app
 
     def ensure_namespace(self):
