@@ -8,13 +8,15 @@ from fastapi.middleware.cors import CORSMiddleware
 from fastapi.responses import RedirectResponse, JSONResponse
 from starlette.middleware.sessions import SessionMiddleware
 from fastapi.templating import Jinja2Templates
-from traitlets import Bool, Unicode, Integer, Type, validate, Instance, default
+from traitlets import Bool, Unicode, Integer, Type, validate, Instance, default, Dict
 from traitlets.config import Application, catch_config_error
 
 from conda_store_server.server import auth, views
 from conda_store_server.app import CondaStore
 from conda_store_server import __version__
 
+import conda_store_server.server.dbutil as dbutil
+
 
 class CondaStoreServer(Application):
     aliases = {
@@ -82,6 +84,12 @@ class CondaStoreServer(Application):
         config=True,
     )
 
+    template_vars = Dict(
+        {},
+        help="Extra variables to be passed into jinja templates for page rendering",
+        config=True,
+    )
+
     @default("templates")
     def _default_templates(self):
         import conda_store_server.server
@@ -120,6 +128,10 @@ class CondaStoreServer(Application):
         self.load_config_file(self.config_file)
 
         self.conda_store = CondaStore(parent=self, log=self.log)
+
+        if self.conda_store.upgrade_db:
+            dbutil.upgrade(self.config.CondaStore.database_url)
+
         self.authentication = self.authentication_class(parent=self, log=self.log)
         # ensure checks on redis_url
         self.conda_store.redis_url
@@ -129,7 +141,7 @@ class CondaStoreServer(Application):
             return url[:-1] if url.endswith("/") else url
 
         app = FastAPI(
-            title="Conda-Store",
+            title="conda-store",
             version=__version__,
             openapi_url=os.path.join(self.url_prefix, "openapi.json"),
             docs_url=os.path.join(self.url_prefix, "docs"),
@@ -153,6 +165,9 @@ class CondaStoreServer(Application):
             SessionMiddleware, secret_key=self.authentication.authentication.secret
         )
 
+        # ensure that template variables are inserted into templates
+        self.templates.env.globals.update(self.template_vars)
+
         @app.middleware("http")
         async def conda_store_middleware(request: Request, call_next):
             try:
