@@ -1,4 +1,5 @@
 from typing import List, Dict, Optional
+import datetime
 
 import pydantic
 import yaml
@@ -131,6 +132,10 @@ def api_get_permissions(
     entity=Depends(dependencies.get_entity),
 ):
     authenticated = entity is not None
+    entity_binding_roles = auth.authorization.get_entity_bindings(
+        entity.role_bindings if authenticated else {}, authenticated=authenticated
+    )
+
     entity_binding_permissions = auth.authorization.get_entity_binding_permissions(
         entity.role_bindings if authenticated else {}, authenticated=authenticated
     )
@@ -146,14 +151,74 @@ def api_get_permissions(
         "status": "ok",
         "data": {
             "authenticated": authenticated,
-            "entity_permissions": entity_binding_permissions,
             "primary_namespace": entity.primary_namespace
             if authenticated
             else conda_store.default_namespace,
+            "entity_permissions": entity_binding_permissions,
+            "entity_roles": entity_binding_roles,
+            "expiration": entity.exp if authenticated else None,
         },
     }
 
 
+@router_api.post(
+    "/token/",
+    response_model=schema.APIPostToken,
+)
+def api_post_token(
+    request: Request,
+    primary_namespace: Optional[str] = Body(None),
+    expiration: Optional[datetime.datetime] = Body(None),
+    role_bindings: Optional[Dict[str, List[str]]] = Body(None),
+    conda_store=Depends(dependencies.get_conda_store),
+    auth=Depends(dependencies.get_auth),
+    entity=Depends(dependencies.get_entity),
+):
+    authenticated = entity is not None
+    current_role_bindings = auth.authorization.get_entity_bindings(
+        entity.role_bindings if authenticated else {}, authenticated=authenticated
+    )
+    current_namespace = (
+        entity.primary_namespace if authenticated else conda_store.default_namespace
+    )
+    current_expiration = (
+        entity.exp
+        if authenticated
+        else (
+            datetime.datetime.now(tz=datetime.timezone.utc) + datetime.timedelta(days=1)
+        )
+    )
+
+    new_namespace = primary_namespace or current_namespace
+    new_role_bindings = role_bindings or current_role_bindings
+    new_expiration = expiration or current_expiration
+
+    if not auth.authorization.is_subset_entity_permissions(
+        current_role_bindings, new_role_bindings, authenticated
+    ):
+        raise HTTPException(
+            status_code=400,
+            detail="Requested role_bindings are not a subset of current permissions",
+        )
+
+    if new_expiration > current_expiration:
+        raise HTTPException(
+            status_code=400,
+            detail="Requested expiration of token is greater than current permissions",
+        )
+
+    token = schema.AuthenticationToken(
+        primary_namespace=new_namespace,
+        role_bindings=new_role_bindings,
+        exp=new_expiration,
+    )
+
+    return {
+        "status": "ok",
+        "data": {"token": auth.authentication.encrypt_token(token)},
+    }
+
+
 @router_api.get(
     "/namespace/",
     response_model=schema.APIListNamespace,
@@ -255,6 +320,9 @@ def api_list_environments(
     search: Optional[str] = None,
     namespace: Optional[str] = None,
     name: Optional[str] = None,
+    status: Optional[schema.BuildStatus] = None,
+    packages: Optional[List[str]] = Query([]),
+    artifact: Optional[schema.BuildArtifactType] = None,
     conda_store=Depends(dependencies.get_conda_store),
     auth=Depends(dependencies.get_auth),
     entity=Depends(dependencies.get_entity),
@@ -267,6 +335,9 @@ def api_list_environments(
             search=search,
             namespace=namespace,
             name=name,
+            status=status,
+            packages=packages,
+            artifact=artifact,
             show_soft_deleted=False,
         ),
     )
@@ -443,13 +514,29 @@ def api_post_specification(
 
 @router_api.get("/build/", response_model=schema.APIListBuild)
 def api_list_builds(
+    status: Optional[schema.BuildStatus] = None,
+    packages: Optional[List[str]] = Query([]),
+    artifact: Optional[schema.BuildArtifactType] = None,
+    environment_id: Optional[int] = None,
+    name: Optional[str] = None,
+    namespace: Optional[str] = None,
     conda_store=Depends(dependencies.get_conda_store),
     auth=Depends(dependencies.get_auth),
     entity=Depends(dependencies.get_entity),
     paginated_args=Depends(get_paginated_args),
 ):
     orm_builds = auth.filter_builds(
-        entity, api.list_builds(conda_store.db, show_soft_deleted=True)
+        entity,
+        api.list_builds(
+            conda_store.db,
+            status=status,
+            packages=packages,
+            artifact=artifact,
+            environment_id=environment_id,
+            name=name,
+            namespace=namespace,
+            show_soft_deleted=True,
+        ),
     )
     return paginated_api_response(
         orm_builds,
@@ -458,6 +545,9 @@ def api_list_builds(
         exclude={"specification", "packages", "build_artifacts"},
         allowed_sort_bys={
             "id": orm.Build.id,
+            "started_on": orm.Build.started_on,
+            "scheduled_on": orm.Build.scheduled_on,
+            "ended_on": orm.Build.ended_on,
         },
         default_sort_by=["id"],
     )
